# @design.estate/dees-element
a custom element class extending lit element class

## Install

To install `@design.estate/dees-element`, you need to run the following command using npm:

```bash
npm install @design.estate/dees-element
```

Ensure you have TypeScript and the necessary compilers configured in your development environment since this package uses ESM syntax and TypeScript.

## Usage

This section provides a comprehensive exploration into the usage of `@design.estate/dees-element`, offering examples to illustrate its capabilities in creating custom elements by extending the lit element class. All examples assume the use of ESM syntax and TypeScript, aligning with modern best practices in web development.

`@design.estate/dees-element` serves as a robust foundation for building custom web components, leveraging the power of the Lit library for reactive updates and efficient rendering management. This guide takes you through the process of creating, styling, and interacting with your custom elements.

### Getting Started

Firstly, ensure you have installed the package as described in the installation section. Once installed, you can start creating custom elements. Let's begin with a simple example:

#### Define a Custom Element

Create a file named `MyElement.ts`:

```typescript
import { DeesElement, customElement, html, css, cssManager } from '@design.estate/dees-element';

@customElement('my-element')
class MyElement extends DeesElement {
  render() {
    return html`<div>My Custom Element</div>`;
  }
}
```

This code snippet defines a custom element `<my-element>` that renders a simple div. `DeesElement` extends the LitElement class, bringing in all of its reactive rendering capabilities along with additional features that `@design.estate/dees-element` offers.

#### Styling the Element

To style the element, utilize the static `styles` property:

```typescript
@customElement('my-styled-element')
class MyStyledElement extends DeesElement {
  static styles = [
    css`
      div {
        color: blue;
        padding: 16px;
        border: 1px solid blue;
      }
    `,
  ];

  render() {
    return html`<div>A Styled Custom Element</div>`;
  }
}
```

The `css` template tag allows you to define CSS styles scoped to your custom element.

#### Leveraging the `cssManager` for Theme Management

`@design.estate/dees-element` introduces a powerful `CssManager` class that facilitates dynamic theming and responsive designs:

```typescript
import { cssManager, DeesElement, customElement, html, css } from '@design.estate/dees-element';

@customElement('themed-element')
class ThemedElement extends DeesElement {
  static styles = [
    cssManager.defaultStyles,
    css`
      :host {
        display: block;
        background-color: var(--theme-background-color, #fff);
      }
    `,
  ];

  constructor() {
    super();
    cssManager.bdTheme('#fff', '#000'); // Define themes
  }

  render() {
    return html`<div>A Themed Custom Element</div>`;
  }
}
```

The `CssManager` allows easy management of themes, breakpoints, and more, ensuring your custom elements remain consistent and responsive across different devices and themes.

### Reactive Properties and State Management

To make your elements interactive and data-driven, `@design.estate/dees-element` offers a seamless approach to managing properties and internal state:

```typescript
@customElement('interactive-element')
class InteractiveElement extends DeesElement {
  @property({ type: String }) labelText = 'Click me';

  @state() private count = 0;

  render() {
    return html`
      <button @click="${this.incrementCount}">${this.labelText}: ${this.count}</button>
    `;
  }

  incrementCount() {
    this.count++;
  }
}
```

Here, `@property` and `@state` decorators are used to reactively update your custom element whenever the internal state or properties change.

### Advanced Scenarios: Directives and Async Data

`@design.estate/dees-element` supports Lit's directives for handling asynchronous data, complex rendering conditions, and more. Here's an example using the `resolve` directive to render content asynchronously:

```typescript
import { resolve } from '@design.estate/dees-element';

@customElement('async-element')
class AsyncElement extends DeesElement {
  private fetchData() {
    return new Promise((resolve) => {
      setTimeout(() => resolve('Data Loaded'), 2000);
    });
  }

  render() {
    return html`${resolve(this.fetchData())}`;
  }
}
```

This example demonstrates how to render async data efficiently, ensuring your custom elements can handle dynamic content and states gracefully.

### Conclusion

`@design.estate/dees-element` enriches the development experience for building custom elements by extending the capabilities of LitElement. Through thoughtful design choices, such as enhanced theming with `CssManager` and the incorporation of useful plugins and decorators, it facilitates a seamless and powerful approach to developing custom web components.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
